/*
 * =====================================================================================
 *
 *       Filename:  main.c
 *
 *    Description start C file for 6502 cpu
 *
 *        Version:  1.0
 *        Created:  2020.09.30 17:43:01
 *       Revision:  none
 *       Compiler:  gcc
 *
 *         Author:  Lucjan Bryndza (LB), lbryndza.p@boff.pl
 *   Organization:  BoFF
 *
 * =====================================================================================
 */


#define LED_REG *(volatile unsigned char*)(0x0400)
#define USART_DR_REG *(volatile unsigned char*)(0x0440)
#define USART_SR_REG *(volatile unsigned char*)(0x0441)
#define USART_CR_REG *(volatile unsigned char*)(0x0442)
#define USART_SR_TXE 0x01
#define USART_SR_RXNE 0x02
#define USART_CR_STX 0x01

static void putch(char ch) {
	while( !(USART_SR_REG & USART_SR_TXE) ) {}
	USART_DR_REG = ch;
	USART_CR_REG |= USART_CR_STX;
}

static unsigned char kbhit(void) {
	return USART_SR_REG & USART_SR_RXNE;
}

static unsigned char getchar(void) {
	while( !(USART_SR_REG & USART_SR_RXNE) ) {};
	return USART_DR_REG;
}

void puts( const char* str ) {
	while(*str) {
		putch(*str++);
	}
}
static void tohex( unsigned char ch ) {
	unsigned char nb1 = ch / 16;
	unsigned char nb2 = ch % 16;
	if(nb1<=9) nb1 += '0';
	else nb1 += 'A' - 10;
	if(nb2<=9) nb2 += '0';
	else nb2 += 'A' - 10;
	putch(nb1);
	putch(nb2);
}

static void led_control(unsigned char led) {
	LED_REG = led;
}

int main() {
	unsigned char ch,lds;
	puts("\r\nHello from 65c02 LED test\r\n");
	puts("Press 1-4 for led toggle\r\n");
	led_control(0xff);
	for(lds=0;;) {
		ch = getchar();
		if(ch<'1' || ch > '4') {
			puts("Error: Unknown key only 1-4 is allowed");
		} else {
			ch -= '0';
			lds ^= 1 << (ch-1);
			led_control(~lds);
			puts("Led: "); tohex(ch); puts(" toggled.\r\n");
		}
	}
	return 0;
}

