
/*
* Main address decoder mapped as
* 0000 - 03FF:  Internal RAM
* 0400 - 04FF:  External peripherals
* 0500 - FBFF:  External peripherals
* FC00 - FFFF:  ROM memory (bootloader)
*/



module main_address_decoder( 
						input [15:0] addr_bus,
					    output rom_cs, 
						output ram_cs,
						output periph_cs
	);

   function inbetween(input [15:0] low, input [15:0] value, input [15:0] high);
		inbetween = (value >= low && value <= high);
   endfunction
   
   function inany(input [15:0] value);
	   inany = (^value===1'bX);
   endfunction

   
   reg [2:0] decoder_al;
   always @(addr_bus) begin
	   (* full_case,parallel_case *)
	   case(1'b1)
		   inany(addr_bus): decoder_al <= 3'b001;
		   inbetween(16'h0000,addr_bus,16'h0FFF): decoder_al <= 3'b001;	//RAM
		   inbetween(16'h0400,addr_bus,16'h04FF): decoder_al <= 3'b010;	//Periph
		   inbetween(16'hE000,addr_bus,16'hFFFF): decoder_al <= 3'b100;	//BootROM
	   endcase
   end
   assign ram_cs = 	  decoder_al[0];
   assign periph_cs = decoder_al[1];
   assign rom_cs =    decoder_al[2];
endmodule

/* Peripheral address decoder 
*  00 - 3F:    Periph1
*  40 - 7F:	   Periph2
*  80 - BF:    Periph3
*  C0 - FF	   Periph4
*/
module device_address_decoder(
	input [7:0] addr_in,
	input periph_cs,
	output periph1,
	output periph2,
	output periph3,
	output periph4
);
	reg [3:0] decoder;
	wire [2:0] addr = { periph_cs, addr_in[7:6] };
	always @(addr) begin
	   (* full_case,parallel_case *)
	   casex(addr)
	   	    3'b100: decoder = 4'b0001;
	   	    3'b101: decoder = 4'b0010;
	   	    3'b110: decoder = 4'b0100;
	   	    3'b111: decoder = 4'b1000;
			default: decoder = 4'b0000;
       endcase
	end
	assign periph1 = decoder[0];
	assign periph2 = decoder[1];
	assign periph3 = decoder[2];
	assign periph4 = decoder[3];
endmodule
