/** 6502 block ram microcontroller internal memory */


module internal_rom_no_tristate
#(parameter BITS=9)
(
	input clk,
	input [BITS-1:0] addr,
	output reg [7:0] data
);
	
	localparam MEM_INIT_FILE="capp.hex";
	reg [7:0] rom_memory[0:(2**BITS-1)];
	initial begin
		if(MEM_INIT_FILE!="")
			$readmemh(MEM_INIT_FILE, rom_memory);
	end
	always @(posedge clk) 
		data <= rom_memory[addr];
endmodule

/* ROM memory */
module internal_rom
#(parameter BITS=10)
( 
	input clk,	// Clock input signal
	input rst,
	input oe,	// Enable signal
	input [BITS-1:0] addr,	//Adress input
	output [7:0] data	// Data output
);
	wire [7:0] ntsdata;
	internal_rom_no_tristate #(.BITS(BITS)) rom(
		.clk(clk),
		.addr(addr),
		.data(ntsdata)
	);
	reg oe_del;
	always @(posedge clk or negedge rst)
		if(~rst) oe_del <= 1'b0;
		else oe_del <= oe;


	tristate_buf tbuf( 
		.oe(oe_del),
		.in(ntsdata),
		.out(data)
	);
endmodule

/* RAM memory */
module internal_ram_no_tristate
#(parameter BITS=10)
(
	input clk,		// Clock input
	input we,		// WR/RD signal
	input [BITS-1:0] addr,	// Address input
	input [7:0] din,		// Data input
	output reg [7:0] dout		//Data output
);
	reg [7:0] ram_memory[0:2**BITS-1];
`ifdef __ICARUS__
	integer i;
	initial begin
		for(i=0;i<(2**BITS-1);++i)
			ram_memory[i] = 0;
	end
`endif
	always @(posedge clk) begin
		if(we) ram_memory[addr] <= din;
		else dout <= ram_memory[addr];
	end
endmodule

module internal_ram
#(parameter BITS=10)
(
	input clk,		// Clock input
	input rst, 		// Reset signal
	input we,		// WR/RD signal
	input oe,		// Chip select signal
	input [BITS-1:0] addr,	// Address input
	input [7:0] din,		// Data input
	output [7:0] dout		//Data output
);
	wire [7:0] idata;
	internal_ram_no_tristate #(.BITS(BITS)) ram(
		.clk(clk),
		.we(we&oe),
		.addr(addr),
		.din(din),
		.dout(idata)
	);

	reg oe_del;
	always @(posedge clk or negedge rst)
		if(~rst) oe_del <= 1'b0;
		else oe_del <= oe;

	tristate_buf tbuf( 
		.oe(oe_del),
		.in(idata),
		.out(dout)
	);
endmodule

