`timescale 1ns/1ps

module usart_tx_testbench();
	reg clk,rst;
	wire clk_rx, clk_tx;
	reg start_tx;
	reg [7:0] tx_data;
	wire tx_pin;
	wire tx_rdy;
	reg [7:0] tx_cnt;

	usart_brr #(.N(15),.K(2416)) usart_presc( 
		.clk_in(clk), .clk_out(clk_rx), .rst(rst)
	);

	usart_tx_div usart_tx_div(.inclk(clk_rx), .rst(rst), .outclk(clk_tx) );

	usart_tx utx( .clk(clk_tx), .rst(rst),
		.en(start_tx), .data(tx_data),
		.tx(tx_pin), .rdy(tx_rdy)
	);

	reg [7:0] tx_buffer[0:256];
	reg [1:0] state,nextstate;
	integer i;
	initial
	begin
		$dumpfile("sim/usarttx_top.lxt");
		$dumpvars(0,utx);
		$dumpvars(0,usart_presc);
		//Uart mealy state
		state <= 2'b0;
		nextstate <= 1'b0;
		for(i=0;i<256;i=i+1) 
			tx_buffer[i] = i;
		start_tx <= 1'b0;
		tx_cnt <= 8'b0;
		clk <= 1'b0;
		rst = 1'b1;
		repeat(10) #200 clk <= ~clk;
		rst = 1'b0;
		repeat(10) #200 clk <= ~clk;
		rst = 1'b1;
		#40;
	end
	
	always #40 clk = ~clk;

	always @(state,tx_rdy)
	(* full_case,parallel_case *)
	case(state)
		0: begin
			if(tx_rdy) begin
				tx_data <= tx_buffer[tx_cnt];
				nextstate <= 1;
				start_tx <= 1;
		    end
		end
		1: begin
			nextstate <= tx_rdy?1:2;
		end
		2: begin
			nextstate <= tx_rdy?3:2;
			start_tx <= tx_rdy?1:0;
		end
		3: begin
			if( tx_rdy ) begin
				nextstate <= 0;
				tx_cnt <= tx_cnt + 1;
			end
		end
	endcase


	always @(posedge clk)
	begin
		if(~rst)  begin
			state <= 0;
		end
		else begin
			state <= nextstate;
			if(tx_cnt>=255) $finish;
		end
	end

endmodule
