/******************************************************************************
*                                                                             *
* Copyright 2016 myStorm Copyright and related                                *
* rights are licensed under the Solderpad Hardware License, Version 0.51      *
* (the “License”); you may not use this file except in compliance with        *
* the License. You may obtain a copy of the License at                        *
* http://solderpad.org/licenses/SHL-0.51. Unless required by applicable       *
* law or agreed to in writing, software, hardware and materials               *
* distributed under this License is distributed on an “AS IS” BASIS,          *
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or             *
* implied. See the License for the specific language governing                *
* permissions and limitations under the License.                              *
*                                                                             *
******************************************************************************/
/*
module chip (
    // 25MHz clock input
    input  clkin,
    // Led outputs
    output [3:0] led,
	// Usart TX
	output usart_tx
  );

  // turn other leds off (active low)
  assign led[0] = 1'b1;
  assign led[1] = 1'b1;
  wire clk;
  wire clk_hi;
  wire rst;

  // Clock and reset generator
  clkgen clk_blk( .clock_in(clkin),
	      .clk_hi(clk_hi),
		  .clk(clk),
		  .rst(rst)
  );


  // Two leds divider
  blink div1 (
    .clk   (clk),
    .led (led[3]),
	.rst(rst)
  );

  blink div2 (
    .clk   (clk_hi),
    .led (led[2]),
	.rst(rst)
  );

  usart_tx_u utest(
	  .clk(clk),
	  .rst(rst),
	  .tx_pin(usart_tx)
  );

endmodule

*/

/*
module chip (
    // 25MHz clock input
    input  clkin,
    // Led outputs
    output [3:0] led,
	// Usart TX
	input usart_tx
  );

  wire clk;
  wire clk_hi;
  wire rst;
  wire [7:0] d8;

  // Clock and reset generator
  clkgen clk_blk( .clock_in(clkin),
	      .clk_hi(clk_hi),
		  .clk(clk),
		  .rst(rst)
  );
  
  // Usart check
  usart_rx_u urx(
	  .clk(clk),
	  .rst(rst),
	  .rxd(usart_tx),
	  .leds(d8)
  );
  assign led = ~d8[3:0];
endmodule
*/

/*
module chip (
    // 25MHz clock input
    input  clkin,
    // Led outputs
    output [3:0] led,
	// Usart TX
	output usart_tx
  );

  // turn other leds off (active low)
  assign led[0] = 1'b1;
  assign led[1] = 1'b1;
  wire clk;
  wire clk_hi;
  wire rst;

  // Clock and reset generator
  clkgen clk_blk( .clock_in(clkin),
	      .clk_hi(clk_hi),
		  .clk(clk),
		  .rst(rst)
  );


  // Two leds divider
  blink div1 (
    .clk   (clk),
    .led (led[3]),
	.rst(rst)
  );

  blink div2 (
    .clk   (clk_hi),
    .led (led[2]),
	.rst(rst)
  );

  usart_tx_with_buffer utest(
	  .clk(clk),
	  .rst(rst),
	  .tx_pin(usart_tx)
  );
endmodule
*/


module chip (
    // 25MHz clock input
    input  clkin,
    // Led outputs
    output [3:0] led,
	// Usart TX
	input usart_tx
  );

  wire clk;
  wire clk_hi;
  wire rst;
  wire [7:0] d8;

  // Clock and reset generator
  clkgen clk_blk( .clock_in(clkin),
	      .clk_hi(clk_hi),
		  .clk(clk),
		  .rst(rst)
  );
  
  // Usart check
  usart_rx_buf_u urx(
	  .clk(clk),
	  .rst(rst),
	  .rxd(usart_tx),
	  .leds(d8)
  );
  assign led = ~d8[3:0];
endmodule

