/* This is a usart tramsmiter with fifo queue */


/* ROM memory */
module rom_memory(
	input clk, 
	input re, 
	input [6:0] addr,
	output reg [7:0] data
);
	
	localparam MEM_INIT_FILE="test/strings.txt";
	reg [7:0] rom_memory[0:127];
	initial begin
		if(MEM_INIT_FILE!="")
			$readmemh(MEM_INIT_FILE, rom_memory);
	end

	always @(posedge clk) 
		if(re) data <= rom_memory[addr];

endmodule 

/* Counter with count and stop */
module counter_up_and_stop( input clk, input rst1, input ce,
						   input rst2, output overflow
);
	reg [24:0] counter;
	assign overflow = counter[24];
	always @(posedge clk or negedge rst1) begin
		 if( ~rst1 ) counter <= 24'b0;
		 else begin 
			 if(rst2) counter <= 24'b0;
			 else if(~overflow && ce) counter <= counter + 1'b1;
		 end
	end
endmodule

module usart_tx_with_buffer(
	input rst,
	input clk,
	output tx_pin
);
 	reg [6:0] rom_address;	// ROM address counter
	reg rom_rd;				// ROM strobe signal
	wire [7:0] u_data;		// Data bus
	reg b_valid;			// Data valid
	wire b_ready;			// Ready
	reg timer_enable;		// Timer enable
	reg timer_reset;		// Timer reset
	wire timer_overflow;	// Overflow signal

	localparam SLEN = 26;
	rom_memory rom(
		.clk(clk),
		.re(rom_rd),
		.addr(rom_address),
		.data(u_data)
	);
	
	usart_tx_fifo utx( .clk(clk), .rst(rst),
		.valid(b_valid), .ready(b_ready),
		.data(u_data), .u_tx_pin(tx_pin)
	);

	counter_up_and_stop timer(
		.rst1(rst), .clk(clk), .ce(timer_enable),
		.rst2(timer_reset), .overflow(timer_overflow)
	);
	
	reg [2:0] fstate;
	localparam STATE_MEM_READ = 	 3'd0;
	localparam STATE_MEM_READ_END =  3'd1;
	localparam STATE_PUSH_CHAR_WT =  3'd2;
	localparaom STATE_PUSH_CHAR_FIN = 3'd3;
	localparam STATE_PUSH_CHAR_TIM = 3'd4;
	localparam STATE_TIM_RESET_END = 3'd5;

	wire string_fin = (rom_address==7'b0);

	always @(posedge clk or negedge rst) begin
		if( ~rst ) begin
			rom_rd <= 1'b0;
			rom_address <= 7'b0;
			b_valid <= 1'b0;
			fstate <= 3'b0;
			timer_enable <= 1'b0;
			timer_reset <= 1'b0;
		end
		else begin /*rst*/
			(* full_case,parallel_case *)
			case(fstate)
				STATE_MEM_READ: begin
					rom_rd <= 1'b1;
					fstate <= STATE_MEM_READ_END;
					timer_enable <= 1'b1;
				end
				STATE_MEM_READ_END: begin
					rom_rd <= 1'b0;
					if(rom_address<SLEN) rom_address <= rom_address + 1'b1;
					else rom_address <= 1'b0;
					fstate <= STATE_PUSH_CHAR_WT;
				end
				STATE_PUSH_CHAR_WT: begin
					if(b_ready) begin
						b_valid <= 1'b1;
						fstate <= STATE_PUSH_CHAR_FIN;
					end
				end
				STATE_PUSH_CHAR_FIN: begin
					b_valid <= 1'b0;
					fstate <= STATE_PUSH_CHAR_TIM;
				end
				STATE_PUSH_CHAR_TIM: begin
					if(~string_fin) begin
						fstate <= STATE_MEM_READ;
					end
					else if(timer_overflow) begin
						timer_reset <= 1'b1;
						fstate <= STATE_TIM_RESET_END;
					end
				end
				STATE_TIM_RESET_END: begin
					timer_reset = 1'b0;
					fstate <= STATE_MEM_READ;
				end
			endcase
		end /*rst*/
	end /*always*/


endmodule
